#!/usr/bin/python3

from os import PathLike
import time

from .format import format_string

_msg_types_ncl = {  # message types without colors
    "info": "[info]: ",
    "ok": "[OK]: ",
    "warning": "[WARNING]: ",
    "error": "[ERROR]: "
}

_msg_types = {  # message types with colors
    "info": "[info]: ",
    "ok": format_string("[§dark_green§boldOK§rs]: "),
    "warning": format_string("§yellow§bold[WARNING]: ", auto_rs=False),
    "error": format_string("§red§bold[ERROR]: ", auto_rs=False)
}


class Log:
    def __init__(
        self,
        log_path: str | PathLike = None,
        console_out: bool = True,
        console_colored: bool = True,
        file_colored: bool = True
    ):
        """
        :param log_path: The path to the log file. E.g. "~/cool_project/crashlog.txt"
        :param console_out: Specifies whether the written messages should be also printed to the console.
        :param console_colored: Specifies whether the messages should be printed in color. (e.g. error messages will be
            colored red)
        :param file_colored: Specifies whether the logfile should also contain color codes.
        """

        self.log_path = log_path
        self._console_out = console_out
        self._console_colored = console_colored
        self._file_colored = file_colored

    def write(self, msg: str, msg_type: str = "info", prefix: str = ""):
        """
        Writes a message to the logfile.

        :param msg: The message to be written.
        :param msg_type: The type of the message. (e.g. "info", "error")
        :param prefix: An optional prefix that gets added at the beginning of the message.
        """

        logfile_msg = self._format_msg(msg, msg_type, self._file_colored, prefix) + "\n"

        if self._console_out:
            console_msg = self._format_msg(msg, msg_type, self._console_colored, prefix)

            print(console_msg)

        if self.log_path is not None:
            logfile = open(self.log_path, "a")
            logfile.write(logfile_msg)
            logfile.close()

    def info(self, msg: str, prefix: str=""):
        """
        Shortcut for Log.write(msg, "info", prefix)

        :param msg: The message to be written.
        :param prefix: An optional prefix that gets added at the beginning of the message.
        """

        self.write(msg, "info", prefix)

    def ok(self, msg: str, prefix: str=""):
        """
        Shortcut for Log.write(msg, "ok", prefix)

        :param msg: The message to be written.
        :param prefix: An optional prefix that gets added at the beginning of the message.
        """

        self.write(msg, "ok", prefix)

    def warning(self, msg: str, prefix: str=""):
        """
        Shortcut for Log.write(msg, "warning", prefix)

        :param msg: The message to be written.
        :param prefix: An optional prefix that gets added at the beginning of the message.
        """

        self.write(msg, "warning", prefix)

    def error(self, msg: str, prefix: str=""):
        """
        Shortcut for Log.write(msg, "error", prefix)

        :param msg: The message to be written.
        :param prefix: An optional prefix that gets added at the beginning of the message.
        """

        self.write(msg, "error", prefix)

    def _format_msg(self, msg: str, msg_type: str= "info", colored: bool=True, prefix: str= ""):
        if colored:
            formatted_msg = _msg_types[msg_type] + msg
            formatted_msg += format_string("§rs")

        else:
            formatted_msg = _msg_types_ncl[msg_type] + msg

        time_code = time.strftime("[%d/%m/%Y|%H:%M:%S %z]")

        formatted_msg = prefix + time_code + formatted_msg

        return formatted_msg

    def read(self):
        """
        Reads the content of the logfile.

        :return: The content of the logfile as a string.
        """

        if self.log_path is not None:
            log_file = open(self.log_path, "r")
            log_content = log_file.read()
            log_file.close()
            return log_content

        else:
            raise FileNotFoundError("You didn't specify a log path at the log instance definition.")

    def clear(self):
        """
        Completely clears the logfile.
        """

        logfile = open(self.log_path, "w")
        logfile.write("")
        logfile.close()
